'' =================================================================================================
''
''   File....... jm_blinker_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 18 JAN 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30

  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con

  #-1, ON, OFF                                                  ' digital control

  #0, LSBFIRST, MSBFIRST    


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  long  b1cog
  long  b1stack[16]

  long  b2cog
  long  b2stack[16]                                                        
                                                                 
                                                                 
pub main                                                         
                                                                 
  setup                                                          
                                                                 
  repeat
    start_blinker(26, 100, 100, @b1cog, @b1stack)
    time.pause(2000)
    
    start_blinker(27, 200, 300, @b1cog, @b1stack) 
    time.pause(3000)

    start_blinker(26,  50,  50, @b1cog, @b1stack)   
    start_blinker(27, 100, 400, @b2cog, @b2stack)
    time.pause(5000)

    stop_blinker(@b1cog)
    stop_blinker(@b2cog)
    time.pause(1000)

                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal *
                                                                 

con

  { Embedded Blinker Object }
  
  
pri start_blinker(pin, onms, offms, p_cog, p_stack)

  stop_blinker(p_cog)

  long[p_cog] := cognew(run_blinker(pin, onms, offms), p_stack) + 1

  return long[p_cog]


pri stop_blinker(p_cog)

  if (long[p_cog])
    cogstop(long[p_cog] - 1)
    long[p_cog] := 0


pri run_blinker(pin, onms, offms) | t

  onms *= MS_001                                                ' convert ms to ticks
  offms *= MS_001

  t := cnt
  repeat
    io.high(pin)
    waitcnt(t += onms)
    io.low(pin)
    waitcnt(t += offms)

  
dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}